#!/bin/bash

# Print each command before running it
set -x

# Exit the script immediately on errors
set -e

# Set some variables for later
PWD="`pwd`"
export TMP="$PWD/tmp/"
export TEMP="$TMP"
export TMPDIR="$TMP"
export TEMPDIR="$TMP"
SVN="$PWD/svn/"
SVNDB="file://$SVN"
SVNDUMP="$PWD/svn-dump"
SVNF="$TMP/svn-filtered/"
SVNCOF="$TMP/svn-co-filtered/"
SVNDBF="file://$SVNF"
SVNDUMPF="$TMP/svn-dump-filtered"
RULES="$PWD/rules"
AUTHORS="$PWD/authors"
AUTHORSAPP="$AUTHORS-append.sh"
FIXPARENT="$PWD/fix-parent.sh"
UNP7Z="$PWD/unpack-7z.sh"

# install deps
printf '=== SECTION #1\n'
sudo apt-get install svn-all-fast-export

# Mount the tmpfs
printf '=== SECTION #2\n'
[[ -d "$TMP" ]] && rm -rf "$TMP"
mkdir "$TMP" || true
#sudo mount -t tmpfs -o size=5g tmpfs "$TMP"
cd "$TMP"

# Download the entire SVN repo locally, saves network traffic for repeat conversions
printf '=== SECTION #3\n'
[[ ! -d "$SVN" ]] && rsync --archive --verbose --progress --compress --hard-links --delete --copy-unsafe-links megaglest.svn.sourceforge.net::svn/megaglest/ "$SVN"

# Pre-cleanup
printf '=== SECTION #4\n'
rm -rf *log* *megaglest*

# Convert the repositories
printf '=== SECTION #5\n'
svn-all-fast-export --identity-map "$AUTHORS" --rules "$RULES"  "$SVN"

# Fix a tag that somehow got broken
printf '=== SECTION #6\n'
cd megaglest-source
git tag 3.2.3 3.2.3^
git branch -D 3.2.3
cd ..

# Graft some bits of history
printf '=== SECTION #7\n'
cd megaglest-data
tail=$(git rev-list --max-parents=0 master | tail -n1)
head=$(git rev-parse backups/master@53)
echo $tail $head > info/grafts
git tag -d backups/master@53
#export tail=$(git rev-list --max-parents=0 master | head -n1)
#export head=$(git rev-list --grep ' updated readme for release' master)
#git filter-branch --prune-empty --parent-filter "$FIXPARENT" -- --all
#git for-each-ref --format="%(refname)" refs/original/ | xargs --no-run-if-empty -n 1 git update-ref -d
#git reflog expire --expire=now --all
cd ..

# Uncompress the 7z files
printf '=== SECTION #8\n'
cd megaglest-data-source
git filter-branch --prune-empty --tree-filter "$UNP7Z" -- --all
git for-each-ref --format="%(refname)" refs/original/ | xargs --no-run-if-empty -n 1 git update-ref -d
cd ..

# Clean up after git-svn
printf '=== SECTION #9\n'
for f in megaglest-source megaglest-data megaglest-data-source ; do
	cd $f

	# Convert the SVN tags/branches to tags
	for t in `git branch | grep -v master` ; do
		git tag $t $t
		git branch -D $t
	done

	git filter-branch --prune-empty -- --all
	git for-each-ref --format="%(refname)" refs/original/ | xargs --no-run-if-empty -n 1 git update-ref -d
	git reflog expire --expire=now --all

	git gc --aggressive --prune=now

	# Add an origin to upload to
	git remote add origin ssh://megaglest.git.sourceforge.net/gitroot/megaglest/$f
	git config branch.master.remote origin
	git config branch.master.merge refs/heads/master
	cd ..
done

printf 'Now enter each directory and run gitk --all to inspect the branches/etc, then run git push'
